<?php

namespace Drupal\liveness\Commands;

use Drush\Commands\DrushCommands;
use GuzzleHttp\Exception\RequestException;

/**
 * Implements event subscriber for handling LivenessCommands events.
 */
class LivenessCommands extends DrushCommands {

  /**
   * Check liveness of environments.
   *
   * @param string|null $url
   *   The URL to check.
   * @param string|null $name
   *   The name of the environment.
   *
   * @command liveness:check
   * @aliases lc
   */
  public function checkLiveness($url = NULL, $name = NULL) {
    if ($url && $name) {
      $this->checkEnvironment($url, $name);
    }
    else {
      $config = $this->config('liveness.settings');
      $environments = $config->get('environments') ?? [];

      foreach ($environments as $environment) {
        if (!empty($environment['enabled'])) {
          $this->checkEnvironment($environment['url'], $environment['name']);
        }
      }
    }
  }

  /**
   * Check a single environment.
   *
   * @param string $url
   *   The URL to check.
   * @param string $name
   *   The name of the environment.
   */
  private function checkEnvironment($url, $name) {
    try {
      $client = $this->httpClient();
      // Break cache with time parameter.
      $response = $client->get($url . '?t=' . time());
      if ($response->getStatusCode() != 200) {
        $this->logEvent($name, 'down');
      }
      else {
        $this->checkAndLogEvent($name, 'up');
      }
    }
    catch (RequestException $e) {
      $this->logEvent($name, 'down');
      $this->logger('liveness')->error('Error checking environment @name: @message', [
        '@name' => $name,
        '@message' => $e->getMessage(),
      ]);
    }
    catch (\Exception $e) {
      $this->logEvent($name, 'down');
      $this->logger('liveness')->error('Error checking environment @name: @message', [
        '@name' => $name,
        '@message' => $e->getMessage(),
      ]);
    }
  }

  /**
   * Log an event.
   *
   * @param string $name
   *   The name of the environment.
   * @param string $status
   *   The status of the environment.
   */
  private function logEvent($name, $status) {
    $log_dir = DRUPAL_ROOT . '/sites/default/files';
    $log_file = $log_dir . '/liveness_log.txt';

    if (!file_exists($log_dir)) {
      mkdir($log_dir, 0777, TRUE);
    }

    $log_entry = date('Y-m-d H:i:s') . " - $name is $status\n";
    file_put_contents($log_file, $log_entry, FILE_APPEND);
    // Ensure the file has rw-rw-rw- permissions.
    chmod($log_file, 0666);

    // Send email notification if the status changes.
    $this->sendEmailNotification($name, $status);

    // Log to dblog.
    $this->logger('liveness')->notice(
      '@name is @status',
      [
        '@name' => $name,
        '@status' => $status,
      ]
    );

  }

  /**
   * Check and log an event if the status changes.
   *
   * @param string $name
   *   The name of the environment.
   * @param string $status
   *   The status of the environment.
   */
  private function checkAndLogEvent($name, $status) {
    $log_dir = DRUPAL_ROOT . '/sites/default/files';
    $log_file = $log_dir . '/liveness_log.txt';

    if (!file_exists($log_file)) {
      $this->logEvent($name, $status);
      return;
    }

    $logs = array_reverse(file($log_file));
    foreach ($logs as $log) {
      if (strpos($log, $name) !== FALSE) {
        if (strpos($log, 'down') !== FALSE && $status === 'up') {
          $this->logEvent($name, 'back up');
        }
        return;
      }
    }

    $this->logEvent($name, $status);
  }

  /**
   * Send email notification when the status changes.
   */
  private function sendEmailNotification($name, $status) {
    $config = $this->config('liveness.settings');
    $notification_emails = $config->get('notification_emails');
    if (empty($notification_emails)) {
      return;
    }

    $emails = array_map('trim', explode(',', $notification_emails));
    $subject = $this->t('Liveness status change for @name', ['@name' => $name]);
    $body = $this->t(
      'The status of @name has changed to @status.',
      [
        '@name' => $name,
        '@status' => $status,
      ]
    );

    foreach ($emails as $email) {
      $mailManager = $this->service('plugin.manager.mail');
      $params = [
        'subject' => $subject,
        'body' => $body,
      ];
      $langcode = $this->currentUser()->getPreferredLangcode();
      $mailManager->mail('liveness', 'status_change', $email, $langcode, $params);
    }
  }

}
